
##################################################################
# TetraMAX Reference Methodology Main Script                     #
# Script: tmax.tcl                                               #
# Version: D-2010.03-SP1 (May 24, 2010)                          #
# Copyright (C) 2008-2010 Synopsys, Inc. All rights reserved.    #
##################################################################

# Option Name:  Setting
# =====================
# Strategy:  TTR
# Transition Delay:  TRUE
# Stuck-at Testing:  TRUE
# Dynamic Bridging:  FALSE
# Static Bridging:  FALSE
# Power-Aware ATPG:  FALSE
# Path Delay:  FALSE
# Lynx-Compatible:  FALSE

##################################################################
#    SETUP: Settings for overall execution                       #
##################################################################

# Log file name is hard-coded to capture setup values. Change the name if desired.
set_messages -log tmax.log -replace
report_version -full
set TMAX_SCRIPTS_ROOT "./rm_tmax_scripts"
set TMAX_SETUP_ROOT "./rm_setup"
# Change TMAX_DEBUG_MODE to TRUE to print full log file for debugging.
set TMAX_DEBUG_MODE FALSE
if { $TMAX_DEBUG_MODE } {
  set_messages -level expert
  source -echo -verbose ${TMAX_SETUP_ROOT}/tmax_setup.tcl
} else {
  source ${TMAX_SETUP_ROOT}/tmax_setup.tcl
}
# END OF tmax_setup.tcl
# Source tmax2pt.tcl which will be used later.
source $env(SYNOPSYS)/auxx/syn/tmax/tmax2pt.tcl

# Uncomment if you want the run to continue if errors are encountered.
#set_commands noabort

# If you have rules to downgrade (with or without masking),
# add them to the variable definitions in tmax_setup.tcl.
if { $RULES_TO_DOWNGRADE != "" } {
   foreach tmax_rule $RULES_TO_DOWNGRADE {
      set_rules $tmax_rule warning
   }
}
if { $RULES_TO_MASK != "" } {
   foreach tmax_rule $RULES_TO_MASK {
      set_rules $tmax_rule warning -mask
   }
}

# If you are NOT using the Unified STIL Flow, this setting is needed for parallel-load simulation.
#set_drc -noshadows
# These additional settings may be needed if M404 warnings are printed.
#set_drc -nodslave_remodel -noreclassify_invalid_dslaves
if { $TMAX_DEBUG_MODE } {
   set_drc -store_setup
   set_pindata -shift_character S
}

# These settings allow more flexible use of fault lists.
set_faults -summary verbose -noequiv_code
# These settings give pessimistic reports, but the information can be useful.
#set_faults -fault_coverage -pt_credit 0

##################################################################
#    BUILD: Read in and build the design                         #
##################################################################

# This setting can be used when the netlist contains escaped identifiers.
# It allows SDC from PrimeTime to be read, but disallows parallel-load simulation.
#set_netlist -escape none
# If two definitions of a module are read, last one read is used.
set_netlist -redefined_module last

# If you have modules with no TMAX models, add them to the variable definition in tmax_setup.tcl.
# The -empty_box switch may be better if tristate buses are involved.
if { $MODULES_TO_BLACK_BOX != "" } {
   foreach module $MODULES_TO_BLACK_BOX {
      set_build -black_box $module
   }
}
# If you have PLL sources coming from modeled instances, add them to the variable definition in tmax_setup.tcl.
if { $INSTANCES_TO_BLACK_BOX != "" } {
   foreach inst $INSTANCES_TO_BLACK_BOX {
      set_build -instance_modify [list $inst TIEX ]
   }
}

# This variable must be defined in tmax_setup.tcl, as there is no default.
foreach tmax_library $TMAX_LIBRARY_FILES {
   read_netlist $tmax_library -library
}
# The default design netlist is the DC-RM output.
# If you are not using the DC-RM, change the variable in tmax_setup.tcl.
foreach tmax_netlist $NETLIST_FILES {
   read_netlist $tmax_netlist
}

run_build_model $DESIGN_NAME

########################################################################
#    TD_DRC: Run DRC for Transition-Delay and/or Dynamic Bridging Test #
########################################################################

# These are the most common transition delay settings.
set_delay -launch_cycle system_clock -nopi_changes -common_launch_capture_clock
add_po_masks -all
add_slow_bidis -all
# Set constraints on the Primary Inputs (top-level inputs) here.
# Normally scan enables and resets must be inactive when testing transition delay faults.
# Uncomment with proper port names and off states.
#add_pi_constraints 0 test_se
#add_pi_constraints 1 reset_n
#add_pi_constraints 0 <input_port>

# If timing exceptions should be read, set the variable in tmax_setup.tcl.
# The flow works best when the set_case_analysis matches the TMAX DRC clocking.
# If this is not the case, change the environment to "sdc_case_analysis".
# By default, hold time exceptions are not used.
# To use hold time exceptions, uncomment the line below.
if { ${EXCEPTION_FILE} != ""} {
   set_sdc -environment tmax_drc
   #set_sdc -hold
   read_sdc $EXCEPTION_FILE
}

report_settings drc
if { $TMAX_DEBUG_MODE } {
   report_settings -all > ${REPORTS_DIR}/${DESIGN_NAME}_trans_drc_settings.rpt
}
# The default STIL protocol file is the DC-RM output.
# If you are not using the DC-RM, change the variable in tmax_setup.tcl.
#
# To update waveform timing, uncomment and edit the script as needed:
#source -echo -verbose ${TMAX_SCRIPTS_ROOT}/tmax_update_spf.tcl
#
# If the STIL contains PatternExecs, one must be specified for run_drc.
# For dynamic testing, usually the test mode name (with OCC enabled) is used.
run_drc $PROTOCOL_FILE
#run_drc $PROTOCOL_FILE -patternexec Internal_scan
# Writing the image file speeds up incremental ATPG or diagnosis runs.
write_image ${RESULTS_DIR}/${DESIGN_NAME}_trans.img -replace -violations -netlist_data

# Uncomment and provide number to use multicore ATPG
# For a quad-core CPU machine for example, set the following
#set_atpg -num_processes 4
# Capture cycles for dynamic testing should be at least 2, or the number of PLL cycles.
set_atpg -capture_cycles 2
set_patterns -histogram_summary
# Uncomment to save shift power at expense of pattern count.
#set_atpg -fill adjacent
# Uncomment and modify to control the chain test pattern.
#set_atpg -chain_test 001100110011C
if { $TMAX_DEBUG_MODE } {
   set_atpg -verbose
}
# After all ATPG settings are made, write constraints for PrimeTime.
# Use -unit ps if you have used update_scale on the waveform tables.
write_timing_constraints ${RESULTS_DIR}/${DESIGN_NAME}_trans_shift.tcl -mode shift -replace
write_timing_constraints ${RESULTS_DIR}/${DESIGN_NAME}_trans_capture.tcl -mode capture -replace -wft _allclock_launch_WFT_ -wft _allclock_capture_WFT_

# Add faults for all fault models to be used for ATPG.
set_faults -persistent_fault_models
set_faults -model transition
add_faults -all
report_summaries faults
set_faults -model stuck
add_faults -all
report_summaries faults

# If Small Delay Defect Testing is used, set the variable in tmax_setup.tcl.
# The max_tmgn and max_delta_per_fault numbers here are good starting values.
# Change them to design-specific values for later runs.
if { ${GLOBAL_SLACK_FILE} != ""} {
   set_faults -model transition
   read_timing ${GLOBAL_SLACK_FILE} -delete
# The starting point for max_tmgn is 20% of the fault list,
# and for max_delta_per_fault is 20% of max_tmgn.
   set_delay -max_tmgn 20%
   set_delay -max_delta_per_fault 20%
}

##################################################################
#    PD_ATPG: Create Patterns for Path-Delay Test                #
##################################################################






##################################################################
#    DB_ATPG: Create Patterns for Dynamic Bridging Fault Test    #
##################################################################






##################################################################
#    TD_ATPG: Create Patterns for Transition-Delay Test          #
##################################################################

set_faults -model transition

report_settings atpg
report_settings simulation
report_settings delay
if { $TMAX_DEBUG_MODE } {
   report_settings -all > ${REPORTS_DIR}/${DESIGN_NAME}_trans_atpg_settings.rpt
}
set_atpg -abort_limit 10
run_atpg -auto
# Additional runs with increased abort limits may improve coverage
#set_atpg -abort_limit 100
#run_atpg -auto
#set_atpg -abort_limit 1000
#run_atpg -auto

# Check simulation results of patterns in debug mode only.
if { $TMAX_DEBUG_MODE } {
   run_simulation
}
write_patterns ${RESULTS_DIR}/${DESIGN_NAME}_trans.bin.gz -format binary -replace -compress gzip
write_patterns ${RESULTS_DIR}/${DESIGN_NAME}_trans.stil.gz -format stil -unified_stil_flow -replace -compress gzip
# If you don't want to use the Unified STIL Flow, uncomment these lines.
#write_patterns ${RESULTS_DIR}/${DESIGN_NAME}_trans_ser.stil.gz -format stil -serial -replace -compress gzip
#write_patterns ${RESULTS_DIR}/${DESIGN_NAME}_trans_par.stil.gz -format stil -parallel -replace -compress gzip
write_faults ${RESULTS_DIR}/${DESIGN_NAME}_trans_post.faults.gz -all -replace -compress gzip

report_clocks -verbose > ${REPORTS_DIR}/${DESIGN_NAME}_trans_clocks.rpt
report_sdc -all_paths -case_values > ${REPORTS_DIR}/${DESIGN_NAME}_trans_exceptions.rpt
report_patterns -all -types > ${REPORTS_DIR}/${DESIGN_NAME}_trans_pat_types.rpt
report_patterns -all -clocking > ${REPORTS_DIR}/${DESIGN_NAME}_trans_pat_clocking.rpt
report_summaries faults patterns cpu mem > ${REPORTS_DIR}/${DESIGN_NAME}_trans_summaries.rpt
if { ${GLOBAL_SLACK_FILE} != ""} {
   report_faults -slack effectiveness > ${REPORTS_DIR}/${DESIGN_NAME}_sddt.rpt
   report_faults -slack tmgn 20 >> ${REPORTS_DIR}/${DESIGN_NAME}_sddt.rpt
   report_faults -slack tdet 20 >> ${REPORTS_DIR}/${DESIGN_NAME}_sddt.rpt
   report_faults -slack delta 20 >> ${REPORTS_DIR}/${DESIGN_NAME}_sddt.rpt
}


##################################################################
#    SA_DRC: Run DRC for Stuck-At and/or Static Bridging Test    #
##################################################################

drc -force
remove_atpg_constraints -all
remove_capture_masks -all
remove_cell_constraints -all
remove_clocks -all
remove_compressors -all
remove_pi_constraints -all
remove_po_masks -all
remove_scan_chains -all
remove_scan_enables -all
remove_slow_bidis -all
remove_slow_cells -all

add_slow_bidis -all
# Set constraints on the Primary Inputs (top-level inputs) here.
# Normally scan enables and resets need no constraints when testing stuck-at faults.
# Uncomment with proper port names and off states.
#add_pi_constraints 0 <input_port>

# To use SDC timing exceptions for stuck-at fault ATPG, uncomment this line:
#set_simulation -timing_exceptions_for_stuck_at
#
# If SDC timing exceptions were used earlier, they do not need to be read again.
# If different exceptions, or a different -hold setting, are used,
# then the old exceptions must be removed before reading new ones.
#remove_sdc -all
#if { ${EXCEPTION_FILE} != ""} {
#   set_sdc -environment tmax_drc
#   #set_sdc -hold
#   read_sdc $EXCEPTION_FILE
#}

report_settings drc
if { $TMAX_DEBUG_MODE } {
   report_settings -all > ${REPORTS_DIR}/${DESIGN_NAME}_stuck_drc_settings.rpt
}
# The default STIL protocol file is the DC-RM output.
# If you are not using the DC-RM, change the variable in tmax_setup.tcl.
# If the STIL contains PatternExecs, one must be specified for run_drc.
# For static testing, usually the test mode name _occ_bypass is used.
run_drc $PROTOCOL_FILE
#run_drc $PROTOCOL_FILE -patternexec Internal_scan_occ_bypass
# Writing the image file speeds up incremental ATPG or diagnosis runs.
write_image ${RESULTS_DIR}/${DESIGN_NAME}_stuck.img -replace -violations -netlist_data

# Uncomment and provide number to use multicore ATPG
# For a quad-core CPU machine for example, set the following
#set_atpg -num_processes 4
# Capture cycles for static testing should be enough to test through memories.
set_atpg -capture_cycles 4
set_patterns -histogram_summary
# Uncomment to save shift power at expense of pattern count.
#set_atpg -fill adjacent
# Uncomment and modify to control the chain test pattern.
#set_atpg -chain_test 001100110011C
if { $TMAX_DEBUG_MODE } {
   set_atpg -verbose
}
# After all ATPG settings are made, write constraints for PrimeTime.
# Use -unit ps if you have used update_scale on the waveform tables.
write_timing_constraints ${RESULTS_DIR}/${DESIGN_NAME}_stuck_shift.tcl -mode shift -replace
write_timing_constraints ${RESULTS_DIR}/${DESIGN_NAME}_stuck_capture.tcl -mode capture -replace


##################################################################
#    SB_ATPG: Create Patterns for Static Bridging Fault Test     #
##################################################################






##################################################################
#    SA_ATPG: Create Patterns for Stuck-At Fault Test            #
##################################################################

set_faults -model stuck
update_faults -direct_credit
report_summaries faults

report_settings atpg
report_settings simulation
if { $TMAX_DEBUG_MODE } {
   report_settings -all > ${REPORTS_DIR}/${DESIGN_NAME}_stuck_atpg_settings.rpt
}
set_atpg -abort_limit 10
run_atpg -auto
# Additional runs with increased abort limits may improve coverage
#set_atpg -abort_limit 100
#run_atpg -auto
#set_atpg -abort_limit 1000
#run_atpg -auto

# Check simulation results of patterns in debug mode only.
if { $TMAX_DEBUG_MODE } {
   run_simulation
}
write_patterns ${RESULTS_DIR}/${DESIGN_NAME}_stuck.bin.gz -format binary -replace -compress gzip
write_patterns ${RESULTS_DIR}/${DESIGN_NAME}_stuck.stil.gz -format stil -unified_stil_flow -replace -compress gzip
# If you don't want to use the Unified STIL Flow, uncomment these lines.
#write_patterns ${RESULTS_DIR}/${DESIGN_NAME}_stuck_ser.stil.gz -format stil -serial -replace -compress gzip
#write_patterns ${RESULTS_DIR}/${DESIGN_NAME}_stuck_par.stil.gz -format stil -parallel -replace -compress gzip
write_faults ${RESULTS_DIR}/${DESIGN_NAME}_stuck_post.faults.gz -all -replace -compress gzip

report_clocks -verbose > ${REPORTS_DIR}/${DESIGN_NAME}_stuck_clocks.rpt
report_patterns -all -types > ${REPORTS_DIR}/${DESIGN_NAME}_stuck_pat_types.rpt
report_summaries faults patterns cpu mem > ${REPORTS_DIR}/${DESIGN_NAME}_stuck_summaries.rpt

##################################################################
#    End of TetraMAX RM main script                              #
##################################################################

report_summaries cpu mem
quit
