
##################################################################
# TetraMAX Reference Methodology Diagnosis Example Script        #
# Script: tmax_diagnosis.tcl                                     #
# Version: D-2010.03-SP1 (May 24, 2010)                          #
# Copyright (C) 2010 Synopsys, Inc. All rights reserved.         #
##################################################################

# This script is an example showing typical diagnosis flows.
# It should be customized as needed.

##################################################################
# SETUP: Read design and pattern data to prepare for diagnosis   #
##################################################################

# Setup variables to be used later:
set_messages -log tmax_diagnosis.log -replace
report_version -full
set TMAX_SCRIPTS_ROOT "./rm_tmax_scripts"
set TMAX_SETUP_ROOT "./rm_setup"
source ${TMAX_SETUP_ROOT}/tmax_setup.tcl

# First, read the image use to generate the pattern being diagnosed:
read_image ${RESULTS_DIR}/${DESIGN_NAME}_trans.img

# Set the pattern file, using the binary pattern set if available:
set_patterns -external ${RESULTS_DIR}/${DESIGN_NAME}_trans.bin.gz

# The failure file is required. To understand the diagnosis flow before
# tester data is available, you can generate failure files in TetraMAX.
# These examples show different fault types, but the bracketed information
# must be replaced by design-specific data:
#run_simulation -failure_file fail_stuck.log -stuck { 0 <pin_pathname> } -replace
#run_simulation -failure_file fail_chain.log -fast { r <chain> <position> } -replace
#run_simulation -subnet [list <n> ] -failure_file fail_subnet.log -stuck { 0 <pin_pathname> } -replace
#run_simulation -failure_file fail_bridge.log -bridge { adom <pin_pathname1> <pin_pathname2> } -replace

# To extract subnet information from the ICC (or MW) database, the MW library
# location and most up-to-date CEL name are needed. This is an example.
# This should only be done once and subnets.txt kept for later use.
read_layout ${DESIGN_NAME}_LIB -cel_name change_names_icc -extract_net all -output subnets.txt

# To diagnose bridging faults, read a bridging node pairs file.
# Diagnosis for bridging can be done even if ATPG did not use bridging faults.
#set_faults -bridge_inputs
#read_layout ${NODE_FILE} -likely_pairs

##################################################################
# DIAGNOSIS: Run diagnosis for routine failures                  #
##################################################################

# Failure files from ATE may not be ideal matches for TetraMAX's expectations.
# It is common to have a cycle offset. If so, provide the number to use:
#set_diagnosis -cycle_offset <integer>
# It is common to report only the first several failures. If the reporting
# limit is known, provide it:
#set_diagnosis -failure_memory_limit <integer>
# If the report is limited but the limit is not known, use this setting:
#set_diagnosis -incomplete_failures

# To limit time spent per diagnosis, either because the design is large
# or you are doing volume diagnosis on many parts, there are 2 choices.
#
# You can set a limit to the number of patterns to be simulated:
#set_diagnosis -sample { <failing_patterns> <passing_patterns> }
# Don't use -sample for chain defects.
#
# Or you can set a time limit which requires "set_commands noabort":
#set_diagnosis -time_limit <seconds>
#set_commands noabort

# Perform diagnosis on the failure file. Multiple runs using different
# failure files with the same pattern may be run in sequence:
run_diagnosis fail1.log > ${REPORTS_DIR}/${DESIGN_NAME}_diagnosis_1.log
run_diagnosis fail2.log > ${REPORTS_DIR}/${DESIGN_NAME}_diagnosis_2.log
run_diagnosis fail3.log > ${REPORTS_DIR}/${DESIGN_NAME}_diagnosis_3.log

##################################################################
# DEBUG: Get more information if the results were unexpected     #
##################################################################

# If a chain failure is suspected, but the chain test is missing or passing,
# the most likely chain defects can be found this way:
foreach_in_collection chain [get_scan_chains -all] {
  set chain_name [get_attribute $chain chain_name]
  run_diagnosis fail.log -assume_chain_defect [list all $chain_name ]
}

# If a timing-related defect is suspected but not diagnosed,
# use this setting to increase the priority of delay defects:
set_diagnosis -delay_type
run_diagnosis fail.log

##################################################################
#    End of TetraMAX RM Diagnosis Example script                 #
##################################################################

report_summaries cpu mem
quit
